//**************************************************
//
// shell.h - Declares a generic GUI shell class
//           for use in CS321 Computer Graphics.
//
//           Basic GUI overhead is provided, but all
//           support for the drawing area, text
//           processing box, file I/O, and custom
//           help screen are to be implemented
//           through a derived class which overrides
//           the virtual methods.
//
// Written by: H. Welch August 2000
//	       Based loosely on preliminary code by Benjamin Stocks
//
// Modified by: H. Welch 26-October-2000
//	       Made the mainlayout widget protected to allow the insertion
//	       of controls and widgets below the command prompt/recall area.
//
// Modified by: H. Welch 24-April-2001
//		Added const to appropriate methods
//
// Modified by: E. Durant 26-September-2003
//		Added std:: as needed for portability
//
// See http://www.msoe.edu/~welch/courses/cs321/shell.html
// for usage details
//
//**************************************************


#ifndef SHELL_H
#define SHELL_H
 
// Other shell related headers
#include <drawarea.h>

// Qt library headers to support the class widgets
#include <qwidget.h>
#include <qcombobox.h>
#include <qtextedit.h>
#include <qstring.h>
#include <qlayout.h>

// Standard system libraries
#include <string>

class Shell : public QWidget
{
Q_OBJECT	// Indicate that this class contains slots

public:	// Constructor and destructor
	Shell (QWidget* parent = NULL, const std::string& name="");
	virtual ~Shell();

protected:
	// Build virtual callbacks to allow for derived behavior
	virtual void Erase();
	virtual void Zoom();
	virtual void Unzoom();
	virtual void Reset();
	virtual bool CmdParse(const std::string& cmd);
	virtual void Draw(QEvent* event);
	virtual void ReadFile(const std::string& file);
	virtual void WriteFile(const std::string& file) const;
	virtual std::string HelpText() const;

	// Method to post resize events to standard callback
	virtual void resizeEvent(QResizeEvent* resizeEvent);

protected slots:
	// Not made virtual to abstract Qt slot particulars from
	// casual users of the shell. This allows basic derived
	// classes to avoid the use of the moc preprocessor.
	
	// File menu and button support
	void OpenSlot();
	void SaveSlot();
	void SaveAsSlot();
	
	// Edit menu and button support
	void EraseSlot();
	void ZoomSlot();
	void UnzoomSlot();
	void ResetSlot();
	
	// Help menu and button support
	void AboutQtSlot();
	void HelpAboutSlot();
	void HelpSlot();
	
	// Drawing area support
	void DrawSlot(QEvent* event);
	
	// Command line support
	void CmdLineSlot();
	void CmdRecallSlot(const QString& oldcmd);
	
private:
	QComboBox* cmdRecall;	// Command recall list
	QLineEdit* cmdLine;	// Command entry area
	QTextEdit* help;

protected:
	DrawArea* drawArea;	// Should be linked with QPainter
			        //  to do actual drawing
	std::string defaultFile;// Default internal file name
				//  so Save doesn't need to prompt
	QVBoxLayout* mainLayout;// Allow derived access to the outer layout
				//  widget to controls can be added below
				//  the command prompt and recall area
};
#endif
