//**************************************************
//
// shell.cpp - Implements a generic GUI shell class
//             for use in CS321 Computer Graphics.
//
//             Basic GUI overhead is provided, but all
//             support for the drawing area, text
//             processing box, file I/O, and custom
//             help screen are to be implemented
//             through a derived class which overrides
//             the virtual methods.
//
// Written by: H. Welch August 2000
//	       Based loosely on preliminary code by Benjamin Stocks
//
// Modified by: H. Welch 26-October-2000
//	       Made the mainlayout widget protected to allow the insertion
//	       of controls and widgets below the command prompt/recall area.
//
// Modified by: H. Welch 24-April-2001
//		Added const to appropriate methods
//
// Modified by: H. Welch 10-May-2001
//		Upgraded namespacing to reflect correct use allowed by new compiler
//
// Modified by: H. Welch September 2002
//		Migrated to Qt 3.0 and changed about documentation path
//
// Modified by: H. Welch October 16, 2002
//		Corrected File Dialog Cancel crash
//
// Modified by: E. Durant September 26, 2003 and October 3, 2003
//		Removed "using" and added std:: prefixes for better portability
//**************************************************

// CS321 class definitions
#include <shell.h>
#include <drawarea.h>

// Qt library headers to support the class widgets
#include <qwidget.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qscrollview.h>
#include <qvbuttongroup.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qstring.h>

// Standard system libraries
#include <string>
#include <iostream>
#include <fstream>
#include <cassert>

//*******************************************************
//
// Shell::Shell - Builds basic GUI by dynamically creating
//                the necessary widgets
//
// Written by: H. Welch August 2000
//
// Modified by: H. Welch 26-October-2000
//	       Made the mainlayout widget protected to allow the insertion
//	       of controls and widgets below the command prompt/recall area.
//
// Paramenters: parent - The widget to draw the shell in 
//                       Default NULL for top level widget
//              name - Title bar for the shell
//
// Returns: NOTHING
//
// Calls:
//	  Qt Library
//        string
//        assert
//
// Globals: qApp
//
//********************************************************

Shell::Shell (QWidget* parent, const std::string& name) :
		QWidget(parent), help(NULL), defaultFile("")
{

	// Display the window name
	setCaption(name.c_str());
	
	// Set the window geometry
	setGeometry(20, 20, 600, 350);
	
	// Build the menus first
	QMenuBar* menuBar = new QMenuBar(this);
	assert(menuBar);
	
	// Start with the file menu
	QPopupMenu* fileMenu = new QPopupMenu();
	assert(fileMenu);
	fileMenu->insertItem("&Open", this, SLOT(OpenSlot()));
	fileMenu->insertItem("&Save", this, SLOT(SaveSlot()));
	fileMenu->insertItem("Save &As", this, SLOT(SaveAsSlot()));
	fileMenu->insertSeparator();
	fileMenu->insertItem("&Quit", qApp, SLOT(quit()));
	menuBar->insertItem("&File", fileMenu);
	
	// Do the edit menu
	QPopupMenu* editMenu = new QPopupMenu();
	assert(editMenu);
	editMenu->insertItem("&Reset", this, SLOT(ResetSlot()));
	editMenu->insertItem("&Zoom", this, SLOT(ZoomSlot()));
	editMenu->insertItem("&Unzoom", this, SLOT(UnzoomSlot()));
	editMenu->insertItem("&Erase", this, SLOT(EraseSlot()));
	menuBar->insertItem("&Edit", editMenu);
	
	// Do the help menu
	QPopupMenu* helpMenu = new QPopupMenu();
	assert(helpMenu);
	helpMenu->insertItem("About &Qt", this, SLOT(AboutQtSlot()));
	helpMenu->insertItem("&About the Shell", this, SLOT(HelpAboutSlot()));
	helpMenu->insertItem("&Using the Shell", this, SLOT(HelpSlot()));
	menuBar->insertSeparator();
	menuBar->insertItem("&Help", helpMenu);
	
	// Create the scrolled drawing area
	drawArea = new DrawArea(this);
	assert(drawArea);
	drawArea->setGeometry(0, 0, 500, 350);
	drawArea->setBackgroundColor("black");
	
	// Connect its drawing events to our generic catching slot
	QObject::connect(drawArea, SIGNAL(drawAreaEvent(QEvent*)),
		         this, SLOT(DrawSlot(QEvent*)));
			 
	// Place it inside a scrolled-view
	QScrollView* scrollArea = new QScrollView(this);
	assert(scrollArea);
	scrollArea->addChild(drawArea);
	
	// Create the command line area
	cmdRecall = new QComboBox(false, this);
	assert(cmdRecall);
	cmdRecall->setFixedWidth(250);
	cmdRecall->setInsertionPolicy(QComboBox::AtTop);
	QObject::connect(cmdRecall, SIGNAL(activated(const QString&)),
			 this, SLOT(CmdRecallSlot(const QString&)));
	
	QLabel* cmdLabel = new QLabel(" cmd> ", this);
	assert(cmdLabel);
	
	cmdLine = new QLineEdit(this);
	assert(cmdLine);
	cmdLine->setFixedWidth(250);
	QObject::connect(cmdLine, SIGNAL(returnPressed()),
			 this, SLOT(CmdLineSlot()));
	
	// Create the button area
	QVButtonGroup* buttons = new QVButtonGroup(this);
	assert(buttons);
	
	// Add buttons to the group and connect them to slots
	QPushButton* resetButton = new QPushButton("Reset", buttons);
	assert(resetButton);
	QObject::connect(resetButton, SIGNAL(clicked()), this, SLOT(ResetSlot()));
	resetButton->setMinimumSize(resetButton->sizeHint());
	
	QPushButton* eraseButton = new QPushButton("Erase", buttons);
	assert(eraseButton);
	QObject::connect(eraseButton, SIGNAL(clicked()), this, SLOT(EraseSlot()));
	eraseButton->setMinimumSize(eraseButton->sizeHint());
	
	QPushButton* zoomButton = new QPushButton("Zoom", buttons);
	assert(zoomButton);
	QObject::connect(zoomButton, SIGNAL(clicked()), this, SLOT(ZoomSlot()));
	zoomButton->setMinimumSize(zoomButton->sizeHint());
	
	QPushButton* unzoomButton = new QPushButton("Unzoom", buttons);
	assert(unzoomButton);
	QObject::connect(unzoomButton, SIGNAL(clicked()), this, SLOT(UnzoomSlot()));
	unzoomButton->setMinimumSize(unzoomButton->sizeHint());
	
	QPushButton* helpButton = new QPushButton("Help", buttons);
	assert(helpButton);
	QObject::connect(helpButton, SIGNAL(clicked()), this, SLOT(HelpSlot()));
	helpButton->setMinimumSize(helpButton->sizeHint());
	
	QPushButton* quitButton = new QPushButton("Quit", buttons);
	assert(quitButton);
	QObject::connect(quitButton, SIGNAL(clicked()), qApp, SLOT(quit()));
	quitButton->setMinimumSize(quitButton->sizeHint());
	
	
	// Layout the GUI using the horizontal and vertical layout widgets
	// Create upper row with drawing area and buttons
	QHBoxLayout* upperLayout = new QHBoxLayout();
	assert(upperLayout);
	upperLayout->addWidget(scrollArea);
	upperLayout->addWidget(buttons);
	
	// Create lower row with command prompt and combo-box
	QHBoxLayout* lowerLayout = new QHBoxLayout();
	assert(lowerLayout);
	lowerLayout->addWidget(cmdLabel);
	lowerLayout->addWidget(cmdLine);
	lowerLayout->addWidget(cmdRecall);
	
	// Create outer vertical layout placing the command line at the bottom
	mainLayout = new QVBoxLayout(this);
	assert(mainLayout);
	mainLayout->addSpacing(menuBar->height());
	mainLayout->addLayout(upperLayout);
	mainLayout->addLayout(lowerLayout);
	
	// Force keyboard focus to the command line
	cmdLine->setFocus();
	
}	
	
//*******************************************************
//
// Shell::~Shell - Shell destructor
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: NONE
//
// Globals: NONE
//
//********************************************************	
		
Shell::~Shell()
{
	// Do nothing
}

	
//*******************************************************
//
// Shell::Erase - virtual method to erase the drawing area
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: iostream
//
// Globals: cout
//
//********************************************************	

void Shell::Erase()
{
	std::cout << "Time to erase the drawing." << std::endl;
}

	
//*******************************************************
//
// Shell::Zoom - virtual method to zoom in on the drawing area
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: iostream
//
// Globals: cout
//
//********************************************************	

void Shell::Zoom()
{
	std::cout << "Time to zoom the drawing." << std::endl;
}
	
//*******************************************************
//
// Shell::Unzoom - virtual method to zoom out the drawing area
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: iostream
//
// Globals: cout
//
//********************************************************	

void Shell::Unzoom()
{
	std::cout << "Time to unzoom the drawing." << std::endl;
}
	
//*******************************************************
//
// Shell::Reset - virtual method to reset the drawing and
//                remove viewing transformations
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: iostream
//
// Globals: cout
//
//********************************************************	

void Shell::Reset()
{
	std::cout << "Time to reset the drawing." << std::endl;
}
	
//*******************************************************
//
// Shell::CmdParse - virtual method to parse a text command
//
// Written by: H. Welch August 2000
//
// Paramenters: cmd - text to parse
//
// Returns: true to log command, false if not to log
//
// Calls: iostream
//        string
//
// Globals: cout
//
//********************************************************	

bool Shell::CmdParse(const std::string& cmd)
{
	std::cout << "Time to parse: " << cmd << std::endl;
	return true; // Log the command
}
	
//*******************************************************
//
// Shell::Draw - virtual method to processing drawing area events
//
// Written by: H. Welch August 2000
//
// Paramenters: event - Detailed drawing area event
//
// Returns: NOTHING
//
// Calls: iostream
//
// Globals: cout
//
//********************************************************	

void Shell::Draw(QEvent* event)
{
	std::cout << "Time to process drawing event of type: " << event->type()
	     << std::endl;
}
	
//*******************************************************
//
// Shell::ReadFile - virtual method to read drawing from file
//
// Written by: H. Welch August 2000
//
// Paramenters: file - path of file to read
//
// Returns: NOTHING
//
// Calls: iostream
//        string
//
// Globals: cout
//
//********************************************************	

void Shell::ReadFile(const std::string& file)
{
	std::cout << "Time to read file: " << file << std::endl;
}

	
//*******************************************************
//
// Shell::WriteFile - virtual method to write drawing from file
//
// Written by: H. Welch August 2000
//
// Paramenters: file - path of file to write to
//
// Returns: NOTHING
//
// Calls: iostream
//        string
//
// Globals: cout
//
//********************************************************	

void Shell::WriteFile(const std::string& file) const
{
	std::cout << "Time to write file: " << file << std::endl;
}
	
//*******************************************************
//
// Shell::HelpText - virtual method to provide derived shell
//                   specific text
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: Help text to be displayed
//
// Calls: string
//
// Globals: NONE
//
//********************************************************	

std::string Shell::HelpText() const
{
	// Text space is limited to 80 chars/line with the default
	// window size having 10 scrolled lines
	std::string retval = "Generic CS321 shell.\n"
	                "Override virtual methods to add functionality.";
			
	return retval;
}
	
	
//*******************************************************
//
// Shell::OpenSlot - Process request to load file using
//                   standard Qt dialog
//
// Written by: H. Welch August 2000
//
// Modified by: H. Welch October 16, 2002
//		Corrected File Dialog Cancel crash
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//        string
//        ReadFile
//
// Globals: NONE
//
//********************************************************	

void Shell::OpenSlot()
{
	// Invoke the Qt file dialog
	QString filename = QFileDialog::getOpenFileName(".", "*.*", this).latin1();
	
	// If a file was specified read it and save its name
	if (!filename.isEmpty()) {
		defaultFile = filename.latin1();
		ReadFile(defaultFile);
	}
}
	
	
//*******************************************************
//
// Shell::SaveSlot - Process request to save file by using
//                   previously specified file or prompt
//                   using Qt dialog if none specified
//
// Written by: H. Welch August 2000
//
// Modified by: H. Welch October 16, 2002
//		Corrected File Dialog Cancel crash
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//	  string
//        WriteFile
//
// Globals: NONE
//
//********************************************************	

void Shell::SaveSlot()
{
	
	// Save to file only if a name exits
	if (defaultFile != "") {
		WriteFile(defaultFile);
	}
	// Otherwise prompt for name using Qt dialog
	else {
		QString filename = QFileDialog::getSaveFileName(".", "*.*", this).latin1();
	
		// If a file was specified save to it and save its name
		if (!filename.isEmpty()) {
			defaultFile = filename.latin1();
			WriteFile(defaultFile);
		}
	}
}

//*******************************************************
//
// Shell::SaveAsSlot - Process request to save file by using
//                     Qt dialog. Prompt for confirmation if the
//                     file already exists.
//
// Written by: H. Welch August 2000
//
// Modified by: H. Welch October 16, 2002
//		Corrected File Dialog Cancel crash
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//        string
//	  fstream
//        WriteFile
//
// Globals: NONE
//
//********************************************************	

void Shell::SaveAsSlot()
{
	
	// Prompt using standard Qt dialog
	QString filename = QFileDialog::getSaveFileName(".", "*.*", this).latin1();
	
	// If a file was specified check it first before proceeding
	if (!filename.isEmpty()) {
		
		// See if the file is empty
		std::ifstream istr(filename.latin1());
		if (istr) {
			// Confirm before proceeding
			std::string fname(filename.latin1());
			std::string warning = "File " + fname + " exists. OK to overwrite?";
			if (QMessageBox::warning(NULL, "File Overwrite Warning",
						 warning.c_str(),
						 "OK", "Cancel", 0, 0, 1) == 0) {
				defaultFile = filename.latin1();
				WriteFile(defaultFile);
			}
		}
		else {
			// Write the file if new
			defaultFile = filename.latin1();
			WriteFile(defaultFile);
		}
	}
}

//*******************************************************
//
// Shell::EraseSlot - Wrapper for erase request. Uses a
//                    separate virtual method to avoid
//                    moc preprocessor need for casual users.
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Erase
//
// Globals: NONE
//
//********************************************************	

void Shell::EraseSlot()
{
	// Chain virtual method
	Erase();
}

//*******************************************************
//
// Shell::ZoomSlot - Wrapper for zoom request. Uses a
//                   separate virtual method to avoid
//                   moc preprocessor need for casual users.
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Zoom
//
// Globals: NONE
//
//********************************************************	

void Shell::ZoomSlot()
{
	// Chain virtual method
	Zoom();
}

//*******************************************************
//
// Shell::UnzoomSlot - Wrapper for unzoom request. Uses a
//                     separate virtual method to avoid
//                     moc preprocessor need for casual users.
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Unzoom
//
// Globals: NONE
//
//********************************************************	

void Shell::UnzoomSlot()
{
	// Chain virtual method
	Unzoom();
}

//*******************************************************
//
// Shell::ResetSlot - Wrapper for reset request. Uses a
//                    separate virtual method to avoid
//                    moc preprocessor need for casual users.
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Reset
//
// Globals: NONE
//
//********************************************************	

void Shell::ResetSlot()
{
	// Chain virtual method
	Reset();
}
	
//*******************************************************
//
// Shell::AboutQtSlot - Invokes about Qt dialog
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//
// Globals: NONE
//
//********************************************************	

void Shell::AboutQtSlot()
{
	// Chain the Qt dialog for this
	QMessageBox::aboutQt(this, "About Qt");
}
	
//*******************************************************
//
// Shell::HelpAboutSlot - Display information about the generic shell
//
// Written by: H. Welch August 2000
//
// Modified by: H. Welch September 2002
//		Migrated to Qt 3.0 and changed about documentation path
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//
// Globals: NONE
//
//********************************************************	

void Shell::HelpAboutSlot()
{
	// Display a messagebox
	QMessageBox::information(this, "About the CS321 Shell",
		"Generic CS321 Base Shell Class\n"
		"Written by: H. Welch August 2000\n"
		"Migrated to Qt 3.0 by H. Welch September 2002\n"
		"For more information on how to override\n"
		"its virtual methods see\n"
		"http://people.msoe.edu/~cs321/shell.shtml");
}
	
//*******************************************************
//
// Shell::HelpSlot - Display help information about the derived shell
//
// Written by: H. Welch August 2000
//
// Modified by: H. Welch September 2002
//	Replaced QMultiLineEdit with QTextEdit for compatibility with Qt 3.0
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//	  HelpText
//
// Globals: NONE
//
//********************************************************	

void Shell::HelpSlot()
{
	// Use a read only text box to allow scrolling
	// See if it hasn't been loaded, create it
	if (!help) {
		help = new QTextEdit(NULL);
		assert(help);
		help->setReadOnly(true);
		help->setCaption("Using this GUI Shell");
		help->setGeometry(100, 100, 500, 150);
		

	}
	
	// Get text to display from virtual method
	help->setText(HelpText().c_str());
	help->show();	
}
	
//*******************************************************
//
// Shell::DrawSlot - Wrapper for drawing area requests. Uses a
//                   separate virtual method to avoid
//                   moc preprocessor need for casual users.
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Draw
//
// Globals: NONE
//
//********************************************************	

void Shell::DrawSlot(QEvent* event)
{
	// Chain the virtual method
	Draw(event);
}
	
//*******************************************************
//
// Shell::resizeEvent - Wrapper for resizing requests to use
//			a single generic drawing area callback.
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Draw
//
// Globals: NONE
//
//********************************************************	

void Shell::resizeEvent(QResizeEvent* resizeEvent)
{
	// Chain the virtual method
	Draw((QEvent*) resizeEvent);
}
	
//*******************************************************
//
// Shell::CmdLineSlot - Gets recently entered command from
//		        user and sends it to the parser
//
// Written by: H. Welch August 2000
//
// Paramenters: NONE
//
// Returns: NOTHING
//
// Calls: Qt Library
//        string
//	  CmdParse
//
// Globals: NONE
//
//********************************************************	

void Shell::CmdLineSlot()
{	
	// Get the command text
	std::string cmd = cmdLine->text().latin1();
	
	// Parse the command
	if (CmdParse(cmd)) {
		// Log the command
		cmdRecall->insertItem(cmd.c_str(), 0);
	}
	
	// Clear the command line
	cmdLine->setText("");
}
	
//*******************************************************
//
// Shell::CmdRecallSlot - Restores a previous command to
//                        the command line for editing
//
// Written by: H. Welch August 2000
//
// Paramenters: oldcmd - The recalled string
//
// Returns: NOTHING
//
// Calls: Qt Library
//        string
//	  CmdParse
//
// Globals: NONE
//
//********************************************************	

void Shell::CmdRecallSlot(const QString& oldcmd)
{
	// Place the text in the command line
	cmdLine->setText(oldcmd);
	
	// Force keyboard focus there
	cmdLine->setFocus();
}
